@@ -1,12 +1,14 @@
 import asyncio
 import functools
+from functools import reduce
 
 from networktools.colorprint import gprint, bprint, rprint
 
 from .taskloop import coromask, renew, simple_fargs, simple_fargs_out
 from .scheduler import TaskScheduler
 from .taskloop import TaskLoop
-
+from rich import print
+from networktools.time import timestamp, now
 
 def get_free_ico(icos):
     ico_list = [key for key, v in icos.items() if not v]
@@ -31,15 +33,14 @@ class TaskAssignator:
     """
 
     def __init__(self, scheduler, queue_tasks, queue_answer, sta_assigned,
-                 dt_status, dt_group, enqueued, locker, *args, **kwargs):
+                 dt_status, dt_group, locker, *args, **kwargs):
         self.scheduler = scheduler
         self.queue_tasks = queue_tasks
         self.queue_ans = queue_answer
         self.sta_assigned = sta_assigned
         self.dt_status = dt_status
         self.dt_group = dt_group
-        self.enqueued = enqueued
-        self.ts = 10
+        self.ts = 5
         self.locker = locker
         self.code_filter = kwargs.get('code_filter', 'code')
         self.counter = 0
@@ -59,10 +60,7 @@ class TaskAssignator:
         """
         await asyncio.sleep(self.ts)
         scheduler = self.scheduler
-        sta_assigned = []
-        for ipt, val in self.sta_assigned.items():
-            for ico, ids in val.items():
-                sta_assigned.append(ids)
+        assigned_ids = scheduler.assigned_ids
         dt_status = self.dt_status
         dt_group = self.dt_group
         msg_in = []
@@ -70,53 +68,60 @@ class TaskAssignator:
             tasks = []
             W = 0
             if not queue_tasks.empty():
+                print(now(), queue_tasks)
                 for i in range(queue_tasks.qsize()):
+                    # receive ids code
                     ids = queue_tasks.get()
+                    code = scheduler.stations.get(ids).get(
+                            self.code_filter)
                     scheduler.status_tasks[ids] = True
                     scheduler.sta_init[ids] = True
-                    if ids in self.enqueued:
-                        q = 0
-                        for ipt in scheduler.proc_tasks.keys():
-                            q += 1
-                            if len(scheduler.proc_tasks.get(ipt)) < \
-                               scheduler.lnproc and \
-                               ids not in sta_assigned:
-                                respuesta = {
-                                    'station': ids,
-                                    'core': ipt,
-                                }
-                                if dt_status == 'GROUP':
-                                    code = scheduler.stations.get(ids).get(
-                                            self.code_filter)
-                                    if code in dt_group:
-                                        icos = scheduler.add_task(ids, ipt)
-                                        scheduler.set_init(ids)
-                                        ico = get_free_ico(icos)
-                                        self.locker.acquire()
-                                        self.scheduler.add_sta_assigned(
-                                            ipt, ico, ids)
-                                        self.locker.release()
-                                        ans = f"TASK {ids} ADDED TO {ipt}, code {code}"
-                                        respuesta.update({'added': True})
-                                        element = scheduler.stations.get(
-                                            ids).get(self.code_filter)
-                                        dt_group.remove(element)
-                                elif dt_status == 'ALL':
-                                    icos = scheduler.add_task(ids, ipt)
-                                    ico = get_free_ico(icos)
-                                    scheduler.set_init(ids)
-                                    qr = {ico: ids}
-                                    self.counter += 1
-                                    ans = "TASK %s ADDED TO %s, de un total de:%d" % (
-                                        ids, ipt, self.counter)
-                                    sta_assigned.append(ids)
-                                    self.locker.acquire()
-                                    self.scheduler.add_sta_assigned(
-                                        ipt, ico, ids)
-                                    self.locker.release()
-                                    respuesta.update({'added': True})
-                                self.queue_ans.put(respuesta)
-                    queue_tasks.task_done()
+                    ipt, available = scheduler.pick_one()
+                    print(now(), ipt, available)
+                    if available:
+                        respuesta = {
+                            'station': ids,
+                            'core': ipt,
+                        }
+                        icos = scheduler.add_task(ids, ipt)
+                        ico = get_free_ico(icos)
+
+                        if dt_status == 'GROUP':
+                            if code in dt_group:
+                                scheduler.set_init(ids)
+                                self.locker.acquire()
+                                self.scheduler.add_sta_assigned(
+                                    ipt, ico, ids)
+                                self.locker.release()
+                                ans = f"TASK {ids} ADDED TO {ipt}, code {code}"
+                                respuesta.update({'added': True})
+                                element = scheduler.stations.get(
+                                    ids).get(self.code_filter)
+                                dt_group.remove(element)
+
+                        elif dt_status == 'ALL':
+                            scheduler.set_init(ids)
+                            qr = {ico: ids}
+                            self.counter += 1
+                            ans = "TASK %s ADDED TO %s, de un total de:%d" % (
+                                ids, ipt, self.counter)
+                            assigned_ids.add(ids)
+                            self.locker.acquire()
+                            print("Adding new station to sta" +\
+                            f"assigned ipt {ipt} -> ico {ico} ->"+\
+                            f"ids {ids}")
+                            print(self.scheduler.assigned_tasks)
+                            self.scheduler.add_sta_assigned(
+                                ipt, ico, ids)
+                            self.locker.release()
+                            respuesta.update({'added':
+                                              True})
+                        self.queue_ans.put(respuesta)
+                    else:
+                        print(now(), "Assignator", "All slots are"+\
+                              f"used, can't add the station {code} ids {ids}")
+
+                queue_tasks.task_done()
             else:
                 pass
                 # self.queue_ans.put({'added': False})
