@@ -1,13 +1,14 @@
 # standar library
 import asyncio
-
+from functools import reduce
 # contrib
 from networktools.library import my_random_string
 from networktools.colorprint import gprint, bprint, rprint
 
+from networktools.time import timestamp, now
 # same module
 from .taskloop import TaskLoop
-
+from rich import print
 
 class TaskScheduler:
     """
@@ -34,7 +35,7 @@ class TaskScheduler:
         self.assigned_tasks = kwargs.get('assigned_tasks', {})
         self.lnproc = kwargs.get('nproc', 3)
         self.sta_init = kwargs.get("sta_init", {})
-        self.enqueued = kwargs.get('enqueued', list())
+        self.enqueued = kwargs.get('enqueued', set())
 
     def set_ipt(self, uin=4):
         """
@@ -283,6 +284,10 @@ class TaskScheduler:
         ico_d[ico] = None
         self.assigned_tasks[ipt] = ico_d
 
+
+    # @property
+    # def ()
+
     def add_task(self, ids, ipt):
         """
         Add an *ids* task to some *ipt* process
@@ -294,6 +299,40 @@ class TaskScheduler:
         icos = self.assigned_tasks[ipt]
         return icos
 
+    @property
+    def assigned_ids(self):
+        assigned_tasks  = set()      # 
+        if self.assigned_tasks:
+            assigned_tasks = reduce(
+                lambda a,b: a|b, 
+                [set(ids_set.values()) for ipt, ids_set in
+                 self.assigned_tasks.items()])
+
+        codes = [self.stations.get(ids, {}).get("code") for ids in assigned_tasks]
+        assigned_ids = set(filter(lambda e:e, codes))
+        return assigned_ids
+
+    @property
+    def availabe_slots(self):
+        print(now(), "Assigned tasks", self.assigned_tasks)
+        available = {}
+        for ipt, tasks in self.assigned_tasks.items():
+            assigned_tasks = {t for t, v in tasks.items()
+                              if v}
+            print(now(), "Available slots assigned task", assigned_tasks)
+            if len(assigned_tasks) == self.lnproc:
+                available[ipt] = False 
+            else:
+                available[ipt] = True 
+        return available
+
+    def pick_one(self):
+        for ipt, val in self.availabe_slots.items():
+            icos = self.assigned_tasks[ipt]
+            if val:
+                return ipt, True
+        return "", False
+
     def manage_tasks(self, ipt):
         """
         A method to manage the tasks assigned to *ipt* process
@@ -306,10 +345,8 @@ class TaskScheduler:
         :param ipt: the key or identifier of a process
         """
         # loop = asyncio.get_event_loop()
-        gprint(f"New ipt task {ipt}")
         loop = asyncio.new_event_loop()
         asyncio.set_event_loop(loop)
-        bprint(f"New loop on ipt {ipt}")
         tasks = []
         self.assigned_tasks[ipt] = {}
         new_dict = {}
